const scheduledIntervalIdentifiers = new Map();
const scheduledTimeoutIdentifiers = new Map();
export const clearScheduledInterval = (timerId) => {
    const identifier = scheduledIntervalIdentifiers.get(timerId);
    if (identifier === undefined) {
        return false;
    }
    clearTimeout(identifier);
    scheduledIntervalIdentifiers.delete(timerId);
    return true;
};
export const clearScheduledTimeout = (timerId) => {
    const identifier = scheduledTimeoutIdentifiers.get(timerId);
    if (identifier === undefined) {
        return false;
    }
    clearTimeout(identifier);
    scheduledTimeoutIdentifiers.delete(timerId);
    return true;
};
const computeDelayAndExpectedCallbackTime = (delay, nowAndTimeOrigin) => {
    const now = performance.now();
    const remainingDelay = delay + nowAndTimeOrigin - now - performance.timeOrigin;
    const expected = now + remainingDelay;
    return { expected, remainingDelay };
};
const setTimeoutCallback = (identifiers, timerId, expected, timerType) => {
    const remainingDelay = expected - performance.now();
    if (remainingDelay > 0) {
        identifiers.set(timerId, setTimeout(setTimeoutCallback, remainingDelay, identifiers, timerId, expected, timerType));
    }
    else {
        identifiers.delete(timerId);
        postMessage({ id: null, method: 'call', params: { timerId, timerType } });
    }
};
export const scheduleInterval = (delay, timerId, nowAndTimeOrigin) => {
    const { expected, remainingDelay } = computeDelayAndExpectedCallbackTime(delay, nowAndTimeOrigin);
    scheduledIntervalIdentifiers.set(timerId, setTimeout(setTimeoutCallback, remainingDelay, scheduledIntervalIdentifiers, timerId, expected, 'interval'));
};
export const scheduleTimeout = (delay, timerId, nowAndTimeOrigin) => {
    const { expected, remainingDelay } = computeDelayAndExpectedCallbackTime(delay, nowAndTimeOrigin);
    scheduledTimeoutIdentifiers.set(timerId, setTimeout(setTimeoutCallback, remainingDelay, scheduledTimeoutIdentifiers, timerId, expected, 'timeout'));
};
//# sourceMappingURL=timer.js.map